/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import org.newdawn.slick.GameContainer;
import java.util.Random;
/**
 * An abstract implementation of the IEnemyAI interface.
 * If executeNextMove is called, it checks if the banana is still moving,
 * or if a throw was already started. If so it returns immediately.
 * Otherwise, it checks weather or not it still needs to think about its
 * next move. If so, it will again do nothing.
 * If it is called and does know what it wants to to, it calls
 * the calculateBeta and calculateVelocity methods (which must be implemented
 * in a sub-class) and starts a throw with the provided values.
 * <p>
 * Currently, it is assumed, that the ai stands to the right of the player.
 * If this is not desired, the MIN and MAX values in this code must be changed.
 *
 * @author sam
 */
public abstract class AbstractEnemyAI implements IEnemyAI {

    /** minimal thinking time the ai should have. */
    protected int MAXTHINKINGTIME = 600;
    /** maximal thinking time the ai should have. */
    protected int MINTHINKINGTIME = 300;

    /** minimal allowed velocity. */
    protected static final float MINVELOCITY = -150.0f;
    /** minimal allowed beta. */
    protected static final float MINBETA = -85f;
    /** maximal allowed velocity. */
    protected static final float MAXVELOCITY = -5.0f;
    /** maximal allowed beta. */
    protected static final float MAXBETA = 0f;

    /** current thinking time. Each time the executeNextMove method is called,
     it will be reduced by one. */
    protected int currentThinking;

    protected GameContainer gc;

    protected BlockMap map;

    /**
     * constructs a new ai that controlls the given banana.
     *
     * @param banana
     */
    public AbstractEnemyAI(BlockMap map) {
        this.map = map;       
        currentThinking = randomBetween(MINTHINKINGTIME, MAXTHINKINGTIME);
    }

    /**
     * if the banana is not moving, it will check if it wants to make a new throw.
     * If so, it will do so with the velocity and beta values provided by
     * the abstract methods in this class.
     *
     * @param time
     * @param gc
     */
    public final void executeNextMove(float time, GameContainer gc) {

        this.gc = gc;
        if (map.getBanana2().isMoving()) {
            return;
        }
        if (!map.getBanana2().isStartedThrow()) {
            if (currentThinking <= 0) {
                calculateVelocityBeta();
                float randomBeta = getBeta();
                float randomVel = getVelocity();
                map.getBanana2().startThrow(randomBeta, randomVel);
                currentThinking = randomBetween(MINTHINKINGTIME, MAXTHINKINGTIME);
            } else {
                currentThinking--;
            }
        }
    }

    /**
     * returns a random integer in between start and end.
     * start and end may be negative, but end must always be bigger than start.
     * If it is not, end - start will be returned
     *
     * @param start
     * @param end
     * @return
     */
    protected static int randomBetween(int start, int end) {
        try {
            Random generator = new Random();

            if (start <= 0 && end <= 0) {
                return (0 - (generator.nextInt(Math.abs(start) - Math.abs(end)) + Math.abs(end)));
            } else if (start > 0 && end > 0) {
                return generator.nextInt(end - start) + start;
            } else if (start <= 0 && end > 0) {
                return (generator.nextInt(end + Math.abs(start))) + start;
            } else {
                System.out.println("randomBetween: problem: start: " + start + " end: " + end);
                return (end - start);
            }
        } catch (Exception e) {
            System.out.println("randomBetween: problem: start: " + start + " end: " + end + " exception: " + e);
            return (end - start);
        }


    }

    /**
     * calculates angles which will be returned by getBeta and getVelocity.
     */
    public abstract void calculateVelocityBeta();

    /**
     * returns an angle that should be used in a throw.
     *
     * @return
     */
    public abstract float getBeta();

    /**
     * returns a velocity that should be used in a throw.
     *
     * @return
     */
    public abstract float getVelocity();
}
