/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Path;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

/**
 * This class represents a Banana (or really any object that may be thrown 
 * in a parabolic trajectory).
 * <p>
 * The actual current position will be saved in the <code>position</code> field
 * of this classes super class (<code>ImageObject</code>).
 * The position passed to the constructor will be used as current actual
 * position as well as spawning point in the future. 
 *
 * @author sam
 */
public class Banana extends CollidableImageObject {

    /** the position right before the last respawn. The y axis goes from top down. */
    private Vector2f positionBeforSpawn; // for ai
    /** the position where this object will be spawned. The y axis goes from top down. */
    private Vector2f spawnPosition;
    /** where the object currently is on a normalized parabolic trajectory
     * (meaning its starting coordinates are (0,0), or the lower left corner. */
    private Vector2f objOnThrowParabel;
    /** the angle in radian the object was thrown at. */
    private float betaRadian;
    private float vx;
    private float vy;

    /** weather or not the object is currently moving. */
    private boolean moving;
    private boolean startedThrow; // for ai

    /**
     * constructs a new banana with given arguments.
     *
     * @param name
     * @param image
     * @param position current actual position as well as future spawning position
     * @param collisionShape
     */
    public Banana(String name, Image image, Vector2f position, Shape collisionShape) {
        super(name, image, position, collisionShape);
        spawnPosition = position.copy();
        objOnThrowParabel = new Vector2f();
        moving = false;
        startedThrow = false;
    }

    /**
     * returns the last actual position before the last respawn.
     * This will be either a position on the border of the screen or
     * an collision element from the map.
     *
     * @return
     */
    public Vector2f getPositionBeforSpawn() {
        return positionBeforSpawn;
    }

    /**
     * returns weather or not a throw has been started.
     * <p>
     * used by the ai.
     *
     * @see IEnemyAI
     * @return
     */
    public boolean isStartedThrow() {
        return startedThrow;
    }

    /**
     * returns weather or not the banana is currently moving.
     *
     * @return
     */
    public boolean isMoving() {
        return moving;
    }

    /**
     * returns the position where this banana will respawn after hitting the border
     * or a collidable object.
     *
     * @return
     */
    public Vector2f getSpawnPosition() {
        return spawnPosition;
    }

    /**
     * returns the banana instantly to its spawn position provided at construction.
     * Also updates <code>moving</code>, <code>startedThrow</code> and <code>positionBeforSpawn</code> fields.
     */
    public void resetBanana() {
        moving = false;
        startedThrow = false;
        positionBeforSpawn = new Vector2f(super.getPosition().x, super.getPosition().y);
        super.setPosition(spawnPosition);
    }

    /**
     * starts a throw in the given angle and with the given velocity.
     * Sets <code>moving</code> and <code>startedThrow</code> true,
     * resets <code>objOnThrowParabel</code> to (0,0), and saves the given
     * arguments in this class.
     * <p>
     * It does not actually carry out the throw. To archive this, see {@link #moveBanana(g, null) moveBanana}.
     *
     * @param g gravity
     * @param betaRadian the angle in radian
     * @param velocity the starting velocity
     */
    public void startThrow(float betaRadian, float velocity) {
        this.moving = true;
        this.objOnThrowParabel.x = 0;
        this.objOnThrowParabel.y = 0;
        this.betaRadian = betaRadian;
        this.vx = velocity * (float) Math.cos((double) betaRadian);
        this.vy = velocity * (float) Math.sin((double) betaRadian);
        this.startedThrow = true;
    }

    // TODO change this, so it actually works with vectors...
    /**
     * moves banana one step further on parabolic trajectory.
     * uses the provided time to calculate the logical next position of the object.
     * If the object moves outside of the screen (provided by <code>GameContainer</code>)
     * the object will be reset to its spawning position.
     *
     * @param time
     * @param gc
     */    
    public void moveBanana(float g, float time, GameContainer gc) {        
        objOnThrowParabel.x = ((float) Math.cos((double) betaRadian)) + vx * time;
        objOnThrowParabel.y = ((float) Math.sin((double) betaRadian)) + vy * time - 0.5f * g * time * time;

        // minus in second arg because 0,0 is upper left for openGL, etc.
        super.setPosition(new Vector2f(spawnPosition.x + objOnThrowParabel.x, spawnPosition.y - objOnThrowParabel.y));

        if (isOutOfScreen(gc)) {
            resetBanana();
        }
    }

    /**
     * returns true if the object left the screen as defined by the
     * <code>getScreenWidth</code>/<code>getScreenHeigh</code>t methods
     * of the <code>GameContainer</code> and false otherwise.
     *
     * @param gc
     * @return
     */
    private boolean isOutOfScreen(GameContainer gc) {
        if (getPosition().x > gc.getScreenWidth() || getPosition().x < 0 || getPosition().y > gc.getScreenHeight()) {
            return true;
        }
        return false;
    }

    // TODO redo this, it is not very efficient.
    /**
     * creates a path representing the current parabolic trajectory.
     * It can be printed on the screen for debug and is also
     * used by some of the implementations of <code>IEnemyAI</code>.
     *
     * @see IEnemyAI
     *
     * @param beta angle (in degrees)
     * @param vel velocity
     * @param gc
     * @return the complete path this banana will fly with given vel and beta
     */    
    public Path getYForX(float g, float beta, float vel, GameContainer gc) {
        Path path = new Path(spawnPosition.x, spawnPosition.y);
        Vector2f objOnThrowParabel;
        float betaRadian;
        float velocity;
        float vx;
        float vy;

        objOnThrowParabel = new Vector2f();
        objOnThrowParabel.x = 0;
        objOnThrowParabel.y = 0;
        betaRadian = (float) Math.toRadians(beta);
        velocity = vel;
        vx = velocity * (float) Math.cos((double) betaRadian);
        vy = velocity * (float) Math.sin((double) betaRadian);

        float currentX = 1, currentY = 1;
        float time = 0.0f;
        int count = 0;
        while (currentX < gc.getScreenWidth() && currentX > 0 && currentY < gc.getScreenHeight()) {
            objOnThrowParabel.x = ((float) Math.cos((double) betaRadian)) + vx * time;
            objOnThrowParabel.y = ((float) Math.sin((double) betaRadian)) + vy * time - 0.5f * g * time * time;
            time += 0.3f;
            currentX = spawnPosition.x + objOnThrowParabel.x;
            currentY = spawnPosition.y - objOnThrowParabel.y;
            path.lineTo(currentX, currentY);
            count++;
            // FIXME loop is off, dirt fix:
            if (count == 150) {
                return path;
            }
        }
        return path;
    }
}
