/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package apes;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Polygon;
import org.newdawn.slick.state.StateBasedGame;

/**
 * A <code>MenuItem</code> represents one part of a <code>Menu</code>.
 * It contains an Image that is shown to represent it when the menu it belongs
 * to is shown. It also contains a <code>MenuAction</code> which contains
 * instruction on how to act if isMouseOverItem returns true and the user
 * pressed a mouse button.
 *
 * @see Menu Menu
 * @see MenuAction MenuAction
 *
 * @author sam
 */
public class MenuItem {

    /** the image, which is shown to represent this MenuItem. */
    private Image img;
    /** the scale at which the image is shown (size). */
    private float scale = 1;
    /** a polygon surrounding the picture. */
    private Polygon polygon;
    /** the amount the scale is change if Item is (de-)selected. */
    private float scaleStep = 0.0001f;
    /** the top left x position of this image.*/
    private float xPos;
    /** the top left y position of this image.*/
    private float yPos;
    /** the action to execute. */
    private MenuAction action;

    /**
     * constructs a new MenuItem.
     *
     * @param img the image, which is shown to represent this MenuItem
     * @param xPos the top left x position of this image
     * @param yPos top left y position of this image
     * @param action the action to execute
     * @param offset the offset, the polygon around this item may have
     */
    public MenuItem(Image img, float xPos, float yPos, float offset, MenuAction action) {
        this.img = img;
        this.xPos = xPos;
        this.yPos = yPos;
        this.polygon = generateSquare(xPos, yPos + offset, img.getWidth(), img.getHeight() / 2);
        this.action = action;
    }

    /**
     * returns true, if the position given by x and y is inside the polygon
     * surrounding this MenuItem.
     *
     * @param x
     * @param y
     * @return
     */
    private boolean isMouseOverItem(float x, float y) {
        return polygon.contains(x, y);
    }

    /**
     * Checks weather or not this MenuItem is affected by the current condition of the mouse.
     * If the mouse is positioned on it, the image representing it will slowly grow
     * to indicate that this item is currently selected.
     * If the mouse is not positioned on it and it is still enlarged, it will shrink.
     * If the mouse is positioned on it, its MenuAction will be executed.
     *
     * @param x position of mouse on x axis
     * @param y position of mouse on y axis
     * @param mouseButtonDown is mouse klicked?
     * @param i delta value
     * @param gc gamecontainer
     * @param sbg statebased game
     * @throws SlickException
     */
    public void applyMouseInput(float x, float y, boolean mouseButtonDown, int i, GameContainer gc, StateBasedGame sbg) throws SlickException {
        if (isMouseOverItem(x, y) && action.isActivated()) {
            if (scale < 1.05f) {
                scale += scaleStep * i;
            }
            if (mouseButtonDown && action.isActivated()) {
                action.execute(gc, sbg, this);
            }
        } else {
            if (scale > 1.0f) {
                scale -= scaleStep * i;
            }
        }
    }

    /**
     * draw the image representing this menuitem with the set x/y position and scale.
     */
    public void render() {
        if (action.isActivated()) {
            img.draw(xPos, yPos, scale);
        } else {
            img.draw(xPos, yPos, new Color(1f, 1f, 1f, 0.5f));
        }

    }

    /**
     * returns the image represented by this item.
     *
     * @return
     */
    public Image getImg() {
        return img;
    }

    /**
     * sets the image to represent this item.
     *
     * @param img
     */
    public void setImg(Image img) {
        this.img = img;
    }

    /**
     * creates a square.
     *
     * @param x top x position
     * @param y top y position
     * @param width
     * @param higth
     * @return
     */
    private Polygon generateSquare(float x, float y, float width, float higth) {
        Polygon p = new Polygon(new float[] {
                    x, y,
                    x, y + higth,
                    x + width, y + higth,
                    x + width, y});
        return p;
    }

    /**
     * returns the polygon surrounding the image representing this item.
     *
     * @return
     */
    public Polygon getPolygon() {
        return polygon;
    }
}
